/*
 * __nlcc_attr_funcnoinit:
 * The function defined this attribute is called by the assembly output,
 * and restrains calling of the builtin memcpy() to initialization of the
 * automatic variables. (See: asm_symbol.c:asm_symbol_function())
 * This attribute is to avoid a recursive call between the assembly output
 * of the builtin memcpy() and other builtin functions.
 * But, initialization of the automatic variables is not processed,
 * you cannot give an initial value to automatic variable of the function
 * defined this attribute.
 *
 * Example:
 *
 * __nlcc_attr_funcnoinit
 * static int __builtin_NG_sample(int arg)
 * {
 *   int ng = 0;  ...not initialized!
 *   ...
 * }
 *
 * __nlcc_attr_funcnoinit
 * static int __builtin_OK_sample(int arg)
 * {
 *   int ok;
 *   ok = 0;
 *   ...
 * }
 */

char *builtin_symbols =
"typedef long *__builtin_va_list;\n"
"long __builtin_va_arg(__nlcc_type_symbol __builtin_va_list arg, __nlcc_type_type int type);\n"
"void __builtin_va_start(__nlcc_type_symbol __builtin_va_list arg, __nlcc_type_symbol int symbol);\n"
"void __builtin_va_end(__nlcc_type_symbol __builtin_va_list arg);\n"
"\n"
"__nlcc_attr_funcnoinit /* used in the processing of assembly output */\n"
"static void *__builtin_memcpy(void *dst, const void *src, int len)\n"
"{\n"
"  int i;\n"
"  char *d;\n"
"  const char *s;\n"
"  d = dst;\n"
"  s = src;\n"
"  for (i = 0; i < len; i++)\n"
"    d[i] = s[i];\n"
"  return dst;\n"
"}\n"
"\n"
"__nlcc_attr_funcnoinit /* used in the processing of assembly output */\n"
"static void *__builtin_memcpy4(void *dst, const void *src, int len)\n"
"{\n"
"  int i;\n"
"  int *d;\n"
"  const int *s;\n"
"  d = dst;\n"
"  s = src;\n"
"  for (i = 0; i < len; i++)\n"
"    d[i] = s[i];\n"
"  return dst;\n"
"}\n"
"\n"
"__nlcc_attr_funcnoinit /* used in the processing of assembly output */\n"
"static void *__builtin_memcpy16(void *dst, const void *src, int len)\n"
"{\n"
"  int i;\n"
"  int *d;\n"
"  const int *s;\n"
"  d = dst;\n"
"  s = src;\n"
"  for (i = 0; i < len; i++) {\n"
"    *(d++) = *(s++);\n"
"    *(d++) = *(s++);\n"
"    *(d++) = *(s++);\n"
"    *(d++) = *(s++);\n"
"  }\n"
"  return dst;\n"
"}\n"
"\n"
"__nlcc_attr_funcnoinit /* used in the processing of assembly output */\n"
"static void *__builtin_bitfield_read(int val, int size, int bits, int shift)\n"
"{\n"
"  int high;\n"
"  high = 32 - (bits + shift);\n"
"  val <<= high;\n"
"  val >>= high + shift;\n"
"  return val;\n"
"}\n"
"\n"
"__nlcc_attr_funcnoinit /* used in the processing of assembly output */\n"
"static void *__builtin_bitfield_uread(unsigned int val, int size, int bits, int shift)\n"
"{\n"
"  int high;\n"
"  high = 32 - (bits + shift);\n"
"  val <<= high;\n"
"  val >>= high + shift;\n"
"  return val;\n"
"}\n"
"\n"
"__nlcc_attr_funcnoinit /* used in the processing of assembly output */\n"
"static void *__builtin_bitfield_write(unsigned int val, unsigned int wval, int size, int bits, int shift)\n"
"{\n"
"  int high;\n"
"  unsigned int mask;\n"
"  mask = ~0;\n"
"  high = 32 - (bits + shift);\n"
"  mask <<= high + shift;\n"
"  mask >>= high;\n"
"  val <<= shift;\n"
"  val &= mask;\n"
"  wval &= ~mask;\n"
"  wval |= val;\n"
"  return wval;\n"
"}\n"
"\n"
"__nlcc_attr_funcnoinit /* used in the processing of assembly output */\n"
"static int __builtin_inv(int val)\n"
"{\n"
"  return val ^ -1;\n"
"}\n"
"\n"
"__nlcc_attr_funcnoinit /* used in the processing of assembly output */\n"
"static int __builtin_minus(int val)\n"
"{\n"
"  return 0 - val;\n"
"}\n"
"\n"
"static unsigned int __builtin_mask32_array[] = {\n"
"  0x00000001, 0x00000002, 0x00000004, 0x00000008,\n"
"  0x00000010, 0x00000020, 0x00000040, 0x00000080,\n"
"  0x00000100, 0x00000200, 0x00000400, 0x00000800,\n"
"  0x00001000, 0x00002000, 0x00004000, 0x00008000,\n"
"  0x00010000, 0x00020000, 0x00040000, 0x00080000,\n"
"  0x00100000, 0x00200000, 0x00400000, 0x00800000,\n"
"  0x01000000, 0x02000000, 0x04000000, 0x08000000,\n"
"  0x10000000, 0x20000000, 0x40000000, 0x80000000,\n"
"};\n"
"\n"
"static unsigned char __builtin_uint_size[] = {\n"
"  sizeof(unsigned int) *  0, sizeof(unsigned int) *  1,\n"
"  sizeof(unsigned int) *  2, sizeof(unsigned int) *  3,\n"
"  sizeof(unsigned int) *  4, sizeof(unsigned int) *  5,\n"
"  sizeof(unsigned int) *  6, sizeof(unsigned int) *  7,\n"
"  sizeof(unsigned int) *  8, sizeof(unsigned int) *  9,\n"
"  sizeof(unsigned int) * 10, sizeof(unsigned int) * 11,\n"
"  sizeof(unsigned int) * 12, sizeof(unsigned int) * 13,\n"
"  sizeof(unsigned int) * 14, sizeof(unsigned int) * 15,\n"
"  sizeof(unsigned int) * 16, sizeof(unsigned int) * 17,\n"
"  sizeof(unsigned int) * 18, sizeof(unsigned int) * 19,\n"
"  sizeof(unsigned int) * 20, sizeof(unsigned int) * 21,\n"
"  sizeof(unsigned int) * 22, sizeof(unsigned int) * 23,\n"
"  sizeof(unsigned int) * 24, sizeof(unsigned int) * 25,\n"
"  sizeof(unsigned int) * 26, sizeof(unsigned int) * 27,\n"
"  sizeof(unsigned int) * 28, sizeof(unsigned int) * 29,\n"
"  sizeof(unsigned int) * 30, sizeof(unsigned int) * 31,\n"
"};\n"
"\n"
"__nlcc_attr_funcnoinit /* used in the processing of assembly output */\n"
"static unsigned int __builtin_mask32(int n)\n"
"{\n"
"  int offset;\n"
"\n"
"  /* return mask32[n]; */ /* Make it declined to avoid multiplication at the pointer calculation */\n"
"\n"
"  /* To avoid a recursive calling of the buildin-left-shift and the buildin-multiplication */\n"
"  offset = __builtin_uint_size[n];\n"
"  return *(unsigned int *)((unsigned long)__builtin_mask32_array + offset);\n"
"}\n"
"\n"
"static unsigned int __builtin_lshlsi3(unsigned int val, int s);\n"
"\n"
"__nlcc_attr_funcnoinit /* used in the processing of assembly output */\n"
"static int __builtin_ashlsi3(int val, int s) /* 32bit left shift arithmetic */\n"
"{\n"
"  return __builtin_lshlsi3(val, s);\n"
"}\n"
"\n"
"__nlcc_attr_funcnoinit /* used in the processing of assembly output */\n"
"static unsigned int __builtin_lshlsi3(unsigned int val, int s) /* 32bit left shift logical */\n"
"{\n"
"  unsigned int v;\n"
"  int i;\n"
"\n"
"  v = 0;\n"
"  s &= (32 - 1);\n"
"\n"
"  for (i = 0; i < 32; i++) {\n"
"    if (i >= s) {\n"
"      if (val & __builtin_mask32(i - s))\n"
"        v |= __builtin_mask32(i);\n"
"    }\n"
"  }\n"
"\n"
"  return v;\n"
"}\n"
"\n"
"__nlcc_attr_funcnoinit /* used in the processing of assembly output */\n"
"static int __builtin_ashrsi3(int val, int s) /* 32bit right shift arithmetic */\n"
"{\n"
"  unsigned int v;\n"
"  int i;\n"
"\n"
"  v = 0;\n"
"  s &= (32 - 1);\n"
"\n"
"  for (i = 0; i < 32; i++) {\n"
"    if (i + s < 32) {\n"
"      if (val & __builtin_mask32(i + s))\n"
"        v |= __builtin_mask32(i);\n"
"    } else {\n"
"      if (val & __builtin_mask32(31))\n"
"        v |= __builtin_mask32(i);\n"
"    }\n"
"  }\n"
"\n"
"  return v;\n"
"}\n"
"\n"
"__nlcc_attr_funcnoinit /* used in the processing of assembly output */\n"
"static unsigned int __builtin_lshrsi3(unsigned int val, int s) /* 32bit right shift logical */\n"
"{\n"
"  unsigned int v;\n"
"  int i;\n"
"\n"
"  v = 0;\n"
"  s &= (32 - 1);\n"
"\n"
"  for (i = 0; i < 32; i++) {\n"
"    if (i + s < 32) {\n"
"      if (val & __builtin_mask32(i + s))\n"
"        v |= __builtin_mask32(i);\n"
"    }\n"
"  }\n"
"\n"
"  return v;\n"
"}\n"
"\n"
"__nlcc_attr_funcnoinit /* used in the processing of assembly output */\n"
"static unsigned int __builtin_div_u32(unsigned int val, unsigned int div, int isrem)\n"
"{\n"
"  unsigned int div2n, quot, rem;\n"
"  unsigned int n;\n"
"\n"
"  if (div == 0)\n"
"    *(int *)0 = 0;\n"
"\n"
"  div2n = div;\n"
"  for (n = 1; ((div2n < val) && ((int)div2n > 0)); n <<= 1)\n"
"    div2n <<= 1;\n"
"\n"
"  rem = val;\n"
"  for (quot = 0; rem >= div; quot += n) {\n"
"    for (; div2n > rem; n >>= 1)\n"
"      div2n >>= 1;\n"
"    rem -= div2n;\n"
"  }\n"
"\n"
"  return isrem ? rem : quot;\n"
"}\n"
"\n"
"__nlcc_attr_funcnoinit /* used in the processing of assembly output */\n"
"static int __builtin_divsi3(int val, int div)\n"
"{\n"
"  int r;\n"
"  int minus;\n"
"\n"
"  minus = 0;\n"
"  if (val < 0) {\n"
"    val = -val;\n"
"    minus = !minus;\n"
"  }\n"
"  if (div < 0) {\n"
"    div = -div;\n"
"    minus = !minus;\n"
"  }\n"
"\n"
"  r = __builtin_div_u32(val, div, 0);\n"
"\n"
"  return minus ? -r : r;\n"
"}\n"
"\n"
"__nlcc_attr_funcnoinit /* used in the processing of assembly output */\n"
"static int __builtin_modsi3(int val, int div)\n"
"{\n"
"  int r;\n"
"  int minus;\n"
"\n"
"  minus = 0;\n"
"  if (val < 0) {\n"
"    val = -val;\n"
"    minus = !minus;\n"
"  }\n"
"  if (div < 0) {\n"
"    div = -div;\n"
"  }\n"
"\n"
"  r = __builtin_div_u32(val, div, 1);\n"
"\n"
"  return minus ? -r : r;\n"
"}\n"
"\n"
"__nlcc_attr_funcnoinit /* used in the processing of assembly output */\n"
"static unsigned int __builtin_udivsi3(unsigned int val, unsigned int div)\n"
"{\n"
"  return __builtin_div_u32(val, div, 0);\n"
"}\n"
"\n"
"__nlcc_attr_funcnoinit /* used in the processing of assembly output */\n"
"static unsigned int __builtin_umodsi3(unsigned int val, unsigned int div)\n"
"{\n"
"  return __builtin_div_u32(val, div, 1);\n"
"}\n"
"\n"
"__nlcc_attr_funcnoinit /* used in the processing of assembly output */\n"
"static unsigned int __builtin_mul_u32(unsigned int val, unsigned int mul)\n"
"{\n"
"  unsigned int val2n, r;\n"
"  unsigned int n;\n"
"\n"
"  val2n = val;\n"
"  for (n = 1; ((n < mul) && ((int)val2n > 0)); n <<= 1)\n"
"    val2n <<= 1;\n"
"\n"
"  for (r = 0; mul > 0; mul -= n) {\n"
"    for (; n > mul; n >>= 1)\n"
"      val2n >>= 1;\n"
"    r += val2n;\n"
"  }\n"
"\n"
"  return r;\n"
"}\n"
"\n"
"__nlcc_attr_funcnoinit /* used in the processing of assembly output */\n"
"static int __builtin_mulsi3(int val,  int mul)\n"
"{\n"
"  int r;\n"
"  int minus;\n"
"\n"
"  minus = 0;\n"
"  if (val < 0) {\n"
"    val = -val;\n"
"    minus = !minus;\n"
"  }\n"
"  if (mul < 0) {\n"
"    mul = -mul;\n"
"    minus = !minus;\n"
"  }\n"
"\n"
"  r = __builtin_mul_u32(val, mul);\n"
"\n"
"  return minus ? -r : r;\n"
"}\n"
"\n"
"__nlcc_attr_funcnoinit /* used in the processing of assembly output */\n"
"static unsigned int __builtin_umulsi3(unsigned int val, unsigned int mul)\n"
"{\n"
"  return __builtin_mulsi3(val, mul);\n"
"}\n"
"\n"
"static unsigned short __builtin_bswap16(unsigned short val)\n"
"{\n"
"  return (val >> 8) | (val << 8);\n"
"}\n"
"static unsigned int __builtin_bswap32(unsigned int val)\n"
"{\n"
"  return __builtin_bswap16(val >> 16) | (__builtin_bswap16(val) << 16);\n"
"}\n"
"static unsigned long long __builtin_bswap64(unsigned long long val)\n"
"{\n"
"  return __builtin_bswap32(val);\n"
"}\n"
"\n"
"# 1\n";
